<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://developers.seeka.co
 * @since      1.0.0
 *
 * @package    Seeka_WooCommerce
 * @subpackage Seeka_WooCommerce/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Seeka_WooCommerce
 * @subpackage Seeka_WooCommerce/includes
 * @author     Seeka <platform@seeka.co>
 */
class Seeka_WooCommerce
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Seeka_WooCommerce_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if (defined('Seeka_WooCommerce_VERSION')) {
			$this->version = Seeka_WooCommerce_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'seeka-woocommerce';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Seeka_WooCommerce_Loader. Orchestrates the hooks of the plugin.
	 * - Seeka_WooCommerce_i18n. Defines internationalization functionality.
	 * - Seeka_WooCommerce_Admin. Defines all hooks for the admin area.
	 * - Seeka_WooCommerce_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-seeka-woocommerce-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-seeka-woocommerce-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-seeka-woocommerce-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-seeka-woocommerce-public.php';

		$this->loader = new Seeka_WooCommerce_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Seeka_WooCommerce_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{

		$plugin_i18n = new Seeka_WooCommerce_i18n();

		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{

		$plugin_admin = new Seeka_WooCommerce_Admin($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');


		// $this->loader->add_action( 'admin_menu', $plugin_admin, 'add_plugin_page' );
		// $this->loader->add_action( 'admin_init', $plugin_admin, 'plugin_page_init' );

		// Enable auto-updates for your plugin.
		$this->loader->add_filter('auto_update_plugin', $plugin_admin, 'enable_auto_updates_for_plugin', 10, 2);

		$this->loader->add_filter('plugins_api', $plugin_admin, 'update_info', 20, 3);
		$this->loader->add_filter('site_transient_update_plugins', $plugin_admin, 'update_exec');
		$this->loader->add_action('upgrader_process_complete', $plugin_admin, 'update_purge', 10, 2);
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Seeka_WooCommerce_Public($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

		// https://www.businessbloomer.com/woocommerce-visual-hook-guide-single-product-page/
		$this->loader->add_action('wp_head', $plugin_public, 'hydrate_seeka_datalayer_handle', -1000);

		$this->loader->add_action('woocommerce_add_to_cart', $plugin_public, 'raise_add_to_cart_handle', 10, 5);

		// https://www.businessbloomer.com/woocommerce-visual-hook-guide-checkout-page/
		// $this->loader->add_action('woocommerce_before_checkout_form', $plugin_public, 'woocommerce_checkout_login_form');

		$this->loader->add_action('woocommerce_before_checkout_form', $plugin_public, 'raise_woocommerce_initcheckout_handle');
		// $this->loader->add_action('woocommerce_before_checkout_billing_form', $plugin_public, 'raise_woocommerce_initcheckout_handle', 10);
		// $this->loader->add_action('woocommerce_after_checkout_form', $plugin_public, 'raise_woocommerce_initcheckout_handle', 10);


		// $this->loader->add_action( 'woocommerce_checkout_order_processed', $plugin_public, 'raise_woocommerce_thankyou_handle', 10, 1 );
		$this->loader->add_action('woocommerce_thankyou', $plugin_public, 'raise_woocommerce_thankyou_handle', 10, 1);
		// $this->loader->add_action( 'woocommerce_after_checkout_form ', $plugin_public, 'raise_woocommerce_thankyou_handle', 10, 1 );

		// $this->loader->add_action( 'woocommerce_checkout_billing', $plugin_public, 'checkout_form_billing' );
		// $this->loader->add_action( 'woocommerce_checkout_shipping', $plugin_public, 'checkout_form_shipping' );

		$this->loader->add_action('wp_ajax_get_product_data', $plugin_public, 'get_product_data', 5, 1);
		$this->loader->add_action('wp_ajax_nopriv_get_product_data', $plugin_public, 'get_product_data', 5, 1);

		// bind check for client side events
		$this->loader->add_action('wp_footer', $plugin_public, 'browser_side_add_events_handle');
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Seeka_WooCommerce_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}
}
