<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://developers.seeka.co
 * @since      1.0.0
 *
 * @package    Seeka_WooCommerce
 * @subpackage Seeka_WooCommerce/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Seeka_WooCommerce
 * @subpackage Seeka_WooCommerce/public
 * @author     Seeka <platform@seeka.co>
 */
class Seeka_WooCommerce_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Seeka_WooCommerce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Seeka_WooCommerce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		// wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/seeka-woocommerce-public.css', array(), $this->version, 'all' );

	}

	function woocommerce_product_to_seeka_product($woo_commerce_product, $cart_item)
	{
		// https://woocommerce.github.io/code-reference/classes/WC-Product.html

		// TODO: handle variants
		// $product_has_variants = $woo_commerce_product->is_type('variation');
		// $product_type = $is_current_product_variant ? 'singleVariant' : 'singleProduct';

		$product_type = 'singleProduct';
		$product_id = $woo_commerce_product->get_id();

		$brands =  wp_get_post_terms($product_id, 'product_brand', ['fields' => 'names']);
		$categories = wp_get_post_terms($product_id, 'product_cat', ['fields' => 'names']);

		return array(
			'productIdentifier' => strval($product_id),
			'productName' => $woo_commerce_product->get_name(),
			'sku' => $woo_commerce_product->get_sku(),
			'brandName' => $brands && !is_wp_error($brands) ? implode(', ', $brands) : null,
			'categoryName' => $categories ? implode(', ', $categories) : null,
			'currencyCode' => get_woocommerce_currency(),
			'type' => $product_type,
			'unitPrice' => $woo_commerce_product->get_display_price(),
			'quantity' => $cart_item ? $cart_item['quantity'] : null,
			'lineItemIdentifier' => $cart_item ? $cart_item['key'] : null,
			// 'variantIdentifier' => json_encode($woo_commerce_product->get_variation_id()),
			// 'variantName' => json_encode($woo_commerce_product->get_variation_description())
		);
	}

	// Add to cart binding
	public function raise_add_to_cart_handle($cart_item_id, $product_id, $quantity, $variation_id, $cart_item_data)
	{
		// https://wp-kama.com/plugin/woocommerce/hook/woocommerce_add_to_cart

		// $current_product = wc_get_product($product_id);
		// if (!$current_product) return;

		// $seeka_product = $this->woocommerce_product_to_seeka_product($current_product, $cart_item_data);
		// $seeka_product['quantity'] = $quantity;

		if (wp_doing_ajax()) {
			// header("X-Seeka-Raise: addToCart");
			// header("X-Seeka-Raise-Args: " . json_encode($seeka_product));
		} else {
			// add_action('wp_head', array($this, 'raise_add_to_cart_event_handle'), -999);

			add_action('wp_footer', function () use ($product_id, $quantity) {
				$is_product_page = is_product();
				if ($is_product_page) {

					$product = wc_get_product($product_id);
					if (!$product) {
						// echo "Product $product_id not found";
						// wp_die();
					} else {
						$seeka_product = $this->woocommerce_product_to_seeka_product($product, null);
						$seeka_product['quantity'] = $quantity;
?>
						<script>
							var c_product_track = <?php echo json_encode($seeka_product) ?>;

							if (Converge) {
								Converge.track.addToCart(c_product_track);
							} else {
								window.addEventListener('converge.sdk.ready', function() {
									Converge.track.addToCart(c_product_track);
								});
							}
						</script>
			<?php
					}
				}
			});
		}
	}

	public function raise_add_to_cart_event_handle($a, $b, $c, $d)
	{
	}

	// Order processed binding
	/*public function raise_checkout_order_processed_handle($order_id)
	{
		add_action('wp_head', array($this, 'raise_checkout_order_processed_event_handle', $order_id), -999);
	}

	public function raise_checkout_order_processed_event_handle($order_id)
	{
		$order = new WC_Order($order_id);
		?>
		<script>
			window.addEventListener('converge.sdk.ready', function() {
				Converge.track.order(<?php echo json_encode($order->get_data()); ?>);
			});
		</script>
		<?php
	}*/

	// Order processed binding
	public function raise_woocommerce_thankyou_handle($order_id)
	{
		// exit();
		//add_action('wp_head', array($this, 'raise_woocommerce_thankyou_event_handle'));
		//  add_action('wp_footer', array($this, 'raise_woocommerce_thankyou_event_handle'));

		//https://woocommerce.github.io/code-reference/classes/WC-Order.html
		add_action('wp_footer', function () use ($order_id) {

			// exit();
			$order = wc_get_order($order_id);
			$customer_id = $order->get_customer_id();
			$seeka_order = array();
			// $seeka_order['cartIdentifier'] = $order->get_id();
			// $seeka_order['checkoutIdentifier'] = $order->;
			$seeka_order['currencyCode'] = $order->get_currency();
			$seeka_order['customerIdentifier'] = $customer_id ? strval($customer_id) : null;
			// $seeka_order['orderIdentifier'] = $order->;
			$seeka_order['orderNumber'] = strval($order_id);
			$seeka_order['paymentMethodName'] = $order->get_payment_method_title();
			$seeka_order['platformType'] = 'wooCommerce';
			$seeka_order['sourceChannelName'] = 'web';
			$seeka_order['totalDiscounts'] = $order->get_discount_total();
			$seeka_order['totalRefunds'] = $order->get_total_refunded();
			$seeka_order['totalShippingPrice'] = $order->get_shipping_total();
			$seeka_order['totalTax'] = $order->get_total_tax();
			$seeka_order['products'] = array();
			foreach ($order->get_items() as $cart_item) {
				// https://woocommerce.github.io/code-reference/classes/WC-Order-Item.html
				$product = $order->get_product_from_item($cart_item);
				$seeka_order['products'][] = $this->woocommerce_product_to_seeka_product($product, $cart_item);
			}

			$seeka_identity = array();
			$seeka_identity['wooCommerce'] = array();

			if ($seeka_order['customerIdentifier']) {
				$seeka_identity['wooCommerce']['customerId'] = array(strval($seeka_order['customerIdentifier']));
			}

			$seeka_identity['wooCommerce']['orderId'] = array(strval($order_id));

			$seeka_identity['firstName'] = array_filter([$order->get_billing_first_name(), $order->get_shipping_first_name()]);
			$seeka_identity['lastName'] = array_filter([$order->get_billing_last_name(), $order->get_shipping_last_name()]);
			$seeka_identity['phone'] = array_filter([$order->get_billing_phone(), $order->get_shipping_phone()]);
			$seeka_identity['email'] = array_filter([$order->get_billing_email()]);

			$seeka_identity['address'] = array();

			// shipping address
			$seeka_identity['address'][] = array();
			$seeka_identity['address'][0]['addressLine1'] = $order->get_shipping_address_1();
			$seeka_identity['address'][0]['addressLine2'] = $order->get_shipping_address_2();
			$seeka_identity['address'][0]['countryCode'] = $order->get_shipping_country();
			$seeka_identity['address'][0]['locality'] = $order->get_shipping_city();
			$seeka_identity['address'][0]['postcode'] = $order->get_shipping_postcode();
			$seeka_identity['address'][0]['stateCode'] = $order->get_shipping_state();
			$hasShippingAddress = true;
			if (!$seeka_identity['address'][0]['addressLine1'] && !$seeka_identity['address'][0]['addressLine2'] && !$seeka_identity['address'][0]['countryCode'] && !$seeka_identity['address'][0]['locality'] && !$seeka_identity['address'][0]['postcode'] && !$seeka_identity['address'][0]['stateCode']) {
				$seeka_identity['address'] = array();
				$hasShippingAddress = false;
			}

			// billing address
			$seeka_identity['address'][] = array();
			$seeka_identity['address'][$hasShippingAddress ? 1 : 0]['addressLine1'] = $order->get_billing_address_1();
			$seeka_identity['address'][$hasShippingAddress ? 1 : 0]['addressLine2'] = $order->get_billing_address_2();
			$seeka_identity['address'][$hasShippingAddress ? 1 : 0]['countryCode'] = $order->get_billing_country();
			$seeka_identity['address'][$hasShippingAddress ? 1 : 0]['locality'] = $order->get_billing_city();
			$seeka_identity['address'][$hasShippingAddress ? 1 : 0]['postcode'] = $order->get_billing_postcode();
			$seeka_identity['address'][$hasShippingAddress ? 1 : 0]['stateCode'] = $order->get_billing_state();

			?>
			<script>
				function _seeka_fire_order_event() {
					Converge.track.order(<?php echo json_encode($seeka_order); ?>);
				}

				function _seeka_fire_identity_track_order() {
					Converge.identity.mergeProfile(<?php echo json_encode($seeka_identity); ?>);
				}

				if (Converge && Converge.config && Converge.status && Converge.status.isInitialised) {
					_seeka_fire_identity_track_order();
				} else {
					window.addEventListener('converge.sdk.ready', function() {
						_seeka_fire_identity_track_order();
					});
				}
				if (Converge) {
					_seeka_fire_order_event();
				} else {
					window.addEventListener('converge.sdk.ready', function() {
						_seeka_fire_order_event();
					});
				}
			</script>
		<?php

		});
	}

	public function get_product_data()
	{
		$product_id = $_POST['product_id'];
		$product = wc_get_product($product_id);
		if (!$product) {
			//   echo "Product $product_id not found";
			// wp_die();
		} else {
			echo json_encode($this->woocommerce_product_to_seeka_product($product, null));
		}
	}

	public function browser_side_add_events_handle()
	{
		// https://wordpress.stackexchange.com/questions/342148/list-of-js-events-in-the-woocommerce-frontend

		?>
		<script>
			jQuery(function() {
				<?php if (is_shop() || is_product_category() || is_product_tag() || is_product()) : // Only for PDP or archives pages 
				?>

					// add to cart
					jQuery('body').on('adding_to_cart', function(ev, a, data, b) {
						try {
							if (window.Seeka && window.Seeka.context && window.Seeka.context.product) {
								window.Seeka.context.product.quantity = data.quantity;
								Converge.track.addToCart(window.Seeka.context.product);
							} else {
								// ajax page trigger or from a product category page
								jQuery.ajax({
									url: '<?php echo admin_url('admin-ajax.php'); ?>',
									type: 'post',
									data: {
										action: 'get_product_data',
										product_id: data.product_id
									},
									dataType: 'json',
									success: function(productData) {
										try {
											productData.quantity = data.quantity;
											Converge.track.addToCart(productData);
										} catch (err) {
											console.error('Seeka', err);
										}
									}
								});
							}
						} catch (err) {
							console.error('Seeka', err);
						}
					});

				<?php endif; ?>

				// remove from cart
				jQuery('body').on('removed_from_cart', function() {
					try {
						Converge.track.removeFromCart();
					} catch (err) {
						console.error('Seeka', err);
					}
				});

				// add payment method
				// payment_method_selected
				jQuery('body').on('init_add_payment_method', function() {
					try {
						Converge.track.addPaymentMethod();
					} catch (err) {
						console.error('Seeka', err);
					}
				});

				// coupon code
				jQuery('body').on('applied_coupon_in_checkout', function(ev, coupon_code) {
					try {
						Converge.track.applyPromotionalCode({
							code: coupon_code
						});
					} catch (err) {
						console.error('Seeka', err);
					}
				});
				jQuery('body').on('applied_coupon', function(ev, coupon_code) {
					try {
						Converge.track.applyPromotionalCode({
							code: coupon_code
						});
					} catch (err) {
						console.error('Seeka', err);
					}
				});
			});
		</script>
		<?php
	}

	// Init checkout binding
	public function raise_woocommerce_initcheckout_handle()
	{
		// die();
		// add_action('wp_footer', array($this, 'raise_woocommerce_initcheckout_event_handle'), -999);

		add_action('wp_footer', function () {
		?>
			<script>
				if (Converge) {
					Converge.track.initiateCheckout(window.Seeka.context.cart);
				} else {
					window.addEventListener('converge.sdk.ready', function() {
						Converge.track.initiateCheckout(window.Seeka.context.cart);
					});
				}
			</script>
			<?php
		});
	}

	// Datalayer
	public function hydrate_seeka_datalayer_handle()
	{
		$is_product_page = is_product();
		// $is_product_category_page = is_product_category();
		// $is_cart_page = is_cart();
		$is_checkout_page = is_checkout();
		$logged_in_user = wp_get_current_user();
		if ($logged_in_user->ID === 0) $logged_in_user = null;

		// Current product detail page
		if ($is_product_page) {
			$current_product = wc_get_product();
			if ($current_product) {
				$seeka_product = $this->woocommerce_product_to_seeka_product($current_product, null);
			?>
				<script>
					window.Seeka = window.Seeka || {};
					window.Seeka.context = window.Seeka.context || {};
					window.Seeka.context.product = <?php echo json_encode($seeka_product); ?>;
				</script>
		<?php
			}
		}

		// tracking defaults
		?>
		<script>
			window.Seeka = window.Seeka || {};
			window.Seeka.context = window.Seeka.context || {};
			window.Seeka.context.tracking = window.Seeka.context.tracking || {};
			window.Seeka.context.tracking.defaults = window.Seeka.context.tracking.defaults || {};
			window.Seeka.context.tracking.defaults.currencyCode = <?php echo json_encode(get_woocommerce_currency()); ?>;
		</script>
		<?php

		/*
		// "quick" add to cart button on product category pages
		$add_to_cart_product_id = isset($_GET['add-to-cart']) && $_GET['add-to-cart'];
		if($add_to_cart_product_id){
			$current_product = wc_get_product($add_to_cart_product_id);
			if($current_product){
				$seeka_product = $this->woocommerce_product_to_seeka_product($current_product);
				?>
					<script>
						window.Seeka = window.Seeka || {};
						window.Seeka.context = window.Seeka.context || {};
						window.Seeka.context.product = <?php echo json_encode($seeka_product); ?>;
					</script>
				<?php
			}
		}*/

		// Search results
		$search_text = isset($_GET['s']) ? $_GET['s'] : null;
		if ($search_text) {
		?>
			<script>
				window.Seeka = window.Seeka || {};
				window.Seeka.context = window.Seeka.context || {};
				window.Seeka.context.search = {
					searchText: <?php echo json_encode($search_text); ?>
				};
			</script>
		<?php
		}

		if ($logged_in_user) {
			// Current logged in user
			$seeka_user = array();
			$seeka_user['email'] = array($logged_in_user->user_email);
			$seeka_user['firstName'] = array($logged_in_user->user_firstname);
			$seeka_user['lastName'] = array($logged_in_user->user_lastname);
			$seeka_user['wooCommerce'] = array();
			$seeka_user['wooCommerce']['customerId'] = array(strval($logged_in_user->ID));
		?>
			<script>
				window.Seeka = window.Seeka || {};
				window.Seeka.context = window.Seeka.context || {};
				window.Seeka.context.user = <?php echo json_encode($seeka_user); ?>;
			</script>
		<?php
		}

		// Current cart
		// https://woocommerce.github.io/code-reference/classes/WC-Cart.html
		global $woocommerce;
		$cart = $woocommerce->cart;
		if ($cart) {
			$seeka_cart = array();
			// $seeka_cart['cartIdentifier'] = $cart->get_id();
			// $seeka_cart['checkoutIdentifier'] = $cart->;
			$seeka_cart['currencyCode'] = get_woocommerce_currency();
			// $seeka_cart['customerIdentifier'] = $cart->;
			// $seeka_cart['orderIdentifier'] = $cart->;
			// $seeka_cart['orderNumber'] = $cart->;
			// $seeka_cart['paymentMethodName'] = $cart->;
			$seeka_cart['platformType'] = 'wooCommerce';
			$seeka_cart['sourceChannelName'] = 'web';
			$seeka_cart['totalDiscounts'] = $cart->get_cart_discount_total();
			// $seeka_cart['totalRefunds'] = $cart->;
			$seeka_cart['totalShippingPrice'] = $cart->get_shipping_total();
			$seeka_cart['totalTax'] = $cart->get_total_tax();
			$seeka_cart['products'] = array();
			foreach ($cart->get_cart() as $cart_item) {
				$seeka_cart['products'][] = $this->woocommerce_product_to_seeka_product($cart_item['data'], $cart_item);
			}
		?>
			<script>
				window.Seeka = window.Seeka || {};
				window.Seeka.context = window.Seeka.context || {};
				window.Seeka.context.cart = <?php echo json_encode($seeka_cart); ?>;
			</script>
<?php
		}
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Seeka_WooCommerce_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Seeka_WooCommerce_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		// wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/seeka-woocommerce-public.js', array( 'jquery' ), $this->version, false );
		// wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/seeka-woocommerce-public.php', array( 'jquery' ), $this->version, false );

		// todo: remove below
		// wp_enqueue_script( $this->plugin_name . 'sidekick', "http://localhost:3000/static/js/bundle.js".'?cache=' . gmdate('U'), array(), $this->version, false );

	}
}
